"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var errors_1 = require("@solana/errors");
var instruction_1 = require("../instruction");
var roles_1 = require("../roles");
var programAddress = 'address';
describe('isInstructionForProgram', function () {
    it('returns true when the instruction has the given program address', function () {
        var instruction = {
            programAddress: programAddress,
        };
        expect((0, instruction_1.isInstructionForProgram)(instruction, programAddress)).toBe(true);
    });
    it('returns false when the instruction does not have the given program address', function () {
        var instruction = {
            programAddress: programAddress,
        };
        var address = 'abc';
        expect((0, instruction_1.isInstructionForProgram)(instruction, address)).toBe(false);
    });
});
describe('assertIsInstructionForProgram', function () {
    it('does not throw when the instruction has the given program address', function () {
        var instruction = {
            programAddress: programAddress,
        };
        var assert = function () { return (0, instruction_1.assertIsInstructionForProgram)(instruction, programAddress); };
        expect(assert).not.toThrow();
    });
    it('throws when the instruction does not have the given program address', function () {
        var instruction = {
            programAddress: programAddress,
        };
        var address = 'abc';
        var assert = function () { return (0, instruction_1.assertIsInstructionForProgram)(instruction, address); };
        expect(assert).toThrow(errors_1.SolanaError);
    });
});
describe('isInstructionWithAccounts', function () {
    it('returns true when the instruction has an array of accounts', function () {
        var instruction = {
            accounts: [
                {
                    address: 'abc',
                    role: roles_1.AccountRole.READONLY,
                },
            ],
            programAddress: programAddress,
        };
        expect((0, instruction_1.isInstructionWithAccounts)(instruction)).toBe(true);
    });
    it('returns true when the instruction has an empty array of accounts', function () {
        var instruction = {
            accounts: [],
            programAddress: programAddress,
        };
        expect((0, instruction_1.isInstructionWithAccounts)(instruction)).toBe(true);
    });
    it('returns false when the instruction does not have accounts defined', function () {
        var instruction = {
            programAddress: programAddress,
        };
        expect((0, instruction_1.isInstructionWithAccounts)(instruction)).toBe(false);
    });
});
describe('assertIsInstructionWithAccounts', function () {
    it('does not throw when the instruction has an array of accounts', function () {
        var instruction = {
            accounts: [
                {
                    address: 'abc',
                    role: roles_1.AccountRole.READONLY,
                },
            ],
            programAddress: programAddress,
        };
        var assert = function () { return (0, instruction_1.assertIsInstructionWithAccounts)(instruction); };
        expect(assert).not.toThrow();
    });
    it('does not throw when the instruction has an empty array of accounts', function () {
        var instruction = {
            accounts: [],
            programAddress: programAddress,
        };
        var assert = function () { return (0, instruction_1.assertIsInstructionWithAccounts)(instruction); };
        expect(assert).not.toThrow();
    });
    it('throws when the instruction does not have accounts defined', function () {
        var instruction = {
            programAddress: programAddress,
        };
        var assert = function () { return (0, instruction_1.assertIsInstructionWithAccounts)(instruction); };
        expect(assert).toThrow(errors_1.SolanaError);
    });
});
describe('isInstructionWithData', function () {
    it('returns true when the instruction has a non-empty data', function () {
        var instruction = {
            data: new Uint8Array([1, 2, 3, 4]),
            programAddress: programAddress,
        };
        expect((0, instruction_1.isInstructionWithData)(instruction)).toBe(true);
    });
    it('returns true when the instruction has an empty data', function () {
        var instruction = {
            data: new Uint8Array([]),
            programAddress: programAddress,
        };
        expect((0, instruction_1.isInstructionWithData)(instruction)).toBe(true);
    });
    it('returns false when the instruction does not have data defined', function () {
        var instruction = {
            programAddress: programAddress,
        };
        expect((0, instruction_1.isInstructionWithData)(instruction)).toBe(false);
    });
});
describe('assertIsInstructionWithData', function () {
    it('does not throw when the instruction has a non-empty data', function () {
        var instruction = {
            data: new Uint8Array([1, 2, 3, 4]),
            programAddress: programAddress,
        };
        var assert = function () { return (0, instruction_1.assertIsInstructionWithData)(instruction); };
        expect(assert).not.toThrow();
    });
    it('does not throw when the instruction has an empty data', function () {
        var instruction = {
            data: new Uint8Array([]),
            programAddress: programAddress,
        };
        var assert = function () { return (0, instruction_1.assertIsInstructionWithData)(instruction); };
        expect(assert).not.toThrow();
    });
    it('throws when the instruction does not have data defined', function () {
        var instruction = {
            programAddress: programAddress,
        };
        var assert = function () { return (0, instruction_1.assertIsInstructionWithData)(instruction); };
        expect(assert).toThrow(errors_1.SolanaError);
    });
});
