"use strict";
/**
 * Quick primer on bitwise operations: https://stackoverflow.com/a/1436448/802047
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.AccountRole = void 0;
exports.downgradeRoleToNonSigner = downgradeRoleToNonSigner;
exports.downgradeRoleToReadonly = downgradeRoleToReadonly;
exports.isSignerRole = isSignerRole;
exports.isWritableRole = isWritableRole;
exports.mergeRoles = mergeRoles;
exports.upgradeRoleToSigner = upgradeRoleToSigner;
exports.upgradeRoleToWritable = upgradeRoleToWritable;
var AccountRole;
(function (AccountRole) {
    // Bitflag guide: is signer ⌄⌄ is writable
    AccountRole[AccountRole["WRITABLE_SIGNER"] = 3] = "WRITABLE_SIGNER";
    AccountRole[AccountRole["READONLY_SIGNER"] = 2] = "READONLY_SIGNER";
    AccountRole[AccountRole["WRITABLE"] = 1] = "WRITABLE";
    AccountRole[AccountRole["READONLY"] = 0] = "READONLY";
})(AccountRole || (exports.AccountRole = AccountRole = {}));
var IS_SIGNER_BITMASK = 2;
var IS_WRITABLE_BITMASK = 1;
function downgradeRoleToNonSigner(role) {
    return role & ~IS_SIGNER_BITMASK;
}
function downgradeRoleToReadonly(role) {
    return role & ~IS_WRITABLE_BITMASK;
}
function isSignerRole(role) {
    return role >= AccountRole.READONLY_SIGNER;
}
function isWritableRole(role) {
    return (role & IS_WRITABLE_BITMASK) !== 0;
}
function mergeRoles(roleA, roleB) {
    return roleA | roleB;
}
function upgradeRoleToSigner(role) {
    return role | IS_SIGNER_BITMASK;
}
function upgradeRoleToWritable(role) {
    return role | IS_WRITABLE_BITMASK;
}
